// where.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

class CError
{
public:
	CError::CError( DWORD dwError = ::GetLastError() ) 
		: 
		m_dwError( dwError )
	{
		::memset( m_szMsg, 0, sizeof m_szMsg );
		::FormatMessage( FORMAT_MESSAGE_IGNORE_INSERTS | 
						 FORMAT_MESSAGE_FROM_SYSTEM |
						 FORMAT_MESSAGE_MAX_WIDTH_MASK,
						 NULL,
						 m_dwError,
						 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
						 m_szMsg,
						 sizeof(m_szMsg)/sizeof(m_szMsg[0]),
						 NULL );
	}

	LPCTSTR Msg() const
	{
		return m_szMsg;
	}

private:
	DWORD m_dwError;
	TCHAR m_szMsg[ 1024 ];
};

#define CMD_VERSIONINFO _T('v')
#define CMD_VERSIONINFOEX _T('x')
#define CMD_SAVE _T('s')
#define CMD_DEFAULT_SETTINGS _T('d')
#define CMD_USAGE _T('?')

namespace {
	const bool DEFAULT_SETTING_VERSIONINFO = false;
	const bool DEFAULT_SETTING_VERSIONINFOEX = false;
	const LPTSTR REGKEY_SHOWVERSION = _T("ShowVersion");
	const LPTSTR REGKEY_SHOWVERBOSEVERSION = _T("ShowVerboseVersion");
};

int GetIntSetting( LPTSTR szKey )
{
	int nValue = 0;
	CRegKey rk;
	if ( ERROR_SUCCESS == rk.Open( HKEY_CURRENT_USER, _T("SOFTWARE\\GlennCarr\\where") ) )
	{
		DWORD dwValue = 0;
		if ( ERROR_SUCCESS == rk.QueryValue( dwValue, szKey ) )
		{
			nValue = dwValue;
		}
	}
	return nValue;
}

int SetIntSetting( LPTSTR szKey, int nValue )
{
	int nStatus = ERROR_SUCCESS;
	CRegKey rk;
	if ( ERROR_SUCCESS == ( nStatus = rk.Create( HKEY_CURRENT_USER, _T("SOFTWARE\\GlennCarr\\where") ) ) )
	{
		DWORD dwValue = nValue;
		if ( ERROR_SUCCESS == ( nStatus = rk.SetValue( dwValue, szKey ) ) )
		{
			nValue = dwValue;
		}
	}
	return nStatus;
}

void ShowUsage( char* argv[] )
{
	char *psz = NULL;
	cout << "\nUsage: " << strtok(( (psz = strrchr(argv[0], '\\'))) ? psz + 1 : argv[0], ".") << 
			" [-v] [-x] [-s] [-d] <executable file>\n" <<
			"\n" <<
			"Finds the file using the rules specified by the SearchPath API method:\n" <<
			"    1. The directory from which the application loaded.\n" <<
			"    2. The current directory.\n" <<
			"    3. The Windows system directory.\n" <<
			"    4. [Windows NT/2000] The 16-bit Windows system directory.\n" <<
			"    5. The Windows directory.\n" <<
			"    6. The directories that are listed in the PATH environment variable.\n" <<
			"It uses .exe, .com, and .cmd as default file types if none is specified.\n" <<
			"See the documentation for the SearchPath API for more details.\n" <<
			"\n" <<
			"Options:\n" <<
			"  -v\tShow version info\n" <<
			"  -x\tShow extended version info\n" <<
			"  -s\tSave command line settings (or lack of) as default\n" <<
			"  -d\tReset default command line settings\n" <<
			"";
}

int main(int argc, char* argv[])
{
	bool bVerboseVersionInfo = (1 == GetIntSetting( REGKEY_SHOWVERBOSEVERSION ) );
	bool bVersionInfo = bVerboseVersionInfo || (1 == GetIntSetting( REGKEY_SHOWVERSION ) );
	bool bSaveForDefault = false;
	bool bDefaultSettings = false;
	int iSearchFileArg = -1;

	for (int iArg = 1; iArg < argc; iArg++ ) {
		switch ( tolower( argv[iArg][0] ) ) {
		case _T('-'):
		case _T('/'):
			switch ( argv[iArg][1] ) {
			case CMD_VERSIONINFO:
				bVersionInfo = true;
				break;
			case CMD_VERSIONINFOEX:
				bVersionInfo = bVerboseVersionInfo = true;
				break;
			case CMD_SAVE:
				bSaveForDefault = true;
				break;
			case CMD_DEFAULT_SETTINGS:
				bDefaultSettings = true;
				break;
			case CMD_USAGE:
				ShowUsage( argv );
				return 0;
			default:
				cout << "Unrecognized option: " << argv[iArg] << "\n";
			}
			break;
		default:
			iSearchFileArg = iArg;
			break;
		}
	}

	if ( iSearchFileArg < 0 )
	{
		ShowUsage( argv );
//		char *psz = NULL;
//		cout << "Usage: " << strtok(( (psz = strrchr(argv[0], '\\'))) ? psz + 1 : argv[0], ".") << 
//				" [-v] [-x] [-s] [-d] <executable file>\n" <<
//				"\n" <<
//				"Finds the file using the rules specified by the SearchPath API method:\n" <<
//				"    1. The directory from which the application loaded.\n" <<
//				"    2. The current directory.\n" <<
//				"    3. The Windows system directory.\n" <<
//				"    4. [Windows NT/2000] The 16-bit Windows system directory.\n" <<
//				"    5. The Windows directory.\n" <<
//				"    6. The directories that are listed in the PATH environment variable.\n" <<
//				"It uses .exe, .com, and .cmd as default file types if none is specified.\n" <<
//				"See the documentation for the SearchPath API for more details.\n" <<
//				"\n" <<
//				"Options:\n" <<
//				"  -v\tShow version info\n" <<
//				"  -x\tShow extended version info\n" <<
//				"  -s\tSave command line settings (or lack of) as default\n" <<
//				"  -d\tReset default command line settings\n" <<
//				"";
		return 0;
	}

	if ( bDefaultSettings )
	{
		bVerboseVersionInfo = DEFAULT_SETTING_VERSIONINFOEX;
		bVersionInfo = DEFAULT_SETTING_VERSIONINFO;
	}

	if ( bSaveForDefault )
	{
		SetIntSetting( REGKEY_SHOWVERSION, bVersionInfo );
		SetIntSetting( REGKEY_SHOWVERBOSEVERSION, bVerboseVersionInfo );
	}

	static LPCTSTR rgszExt[] = { _T(".exe"), _T(".com"), _T(".cmd") };

	TCHAR szPath[ 1024 ];
	LPTSTR pszName = NULL;
	DWORD dwPathLen = 0;
		
	for ( int iExt = 0; (0 == dwPathLen) && iExt < (sizeof(rgszExt)/sizeof(rgszExt[0])); iExt++ ) {
		dwPathLen = ::SearchPath( NULL, 
									   argv[ iSearchFileArg ],
									   rgszExt[ iExt ],
									   sizeof(szPath)/sizeof(szPath[0]),
									   szPath,
									   &pszName );
	}
	CError lastError;
	if ( 0 == dwPathLen ) {
		cerr << lastError.Msg() << endl;
	} else {

		// Display path
		cout << szPath << endl;

		if ( !bVersionInfo )
			return 0;

		DWORD dwHandle = 0;
		DWORD cBytes = ::GetFileVersionInfoSize( szPath, &dwHandle );
		if ( 0 == cBytes )
			cerr << "No version information found." << endl;
		else {
			char* pvi = new char [ cBytes ];
			if ( 0 == ::GetFileVersionInfo( szPath, 0, cBytes, pvi ) )
				cerr << "No version information found." << endl;
			else {
				void* psfi = 0;
				UINT nBytes = 0;

				if ( 0 == ::VerQueryValue( pvi, _T("\\VarFileInfo\\Translation"), &psfi, &nBytes ) )
					cerr << "No version information found." << endl;
				else {
					TCHAR szLang[ 32 ];
					WORD* pw = (WORD*)psfi;
					_stprintf( szLang, "\\StringFileInfo\\%04X%04X\\", pw[ 0 ], pw[ 1 ] );
					static LPTSTR szVar[] = {
						_T("CompanyName"),
						_T("FileDescription"),
						_T("FileVersion"),
						_T("InternalName"),
						_T("LegalCopyright"),
						_T("OriginalFilename"),
						_T("ProductName"),
						_T("ProductVersion"),
						_T("OLESelfRegister"),
           				};
					cout << "\nVersion info:\n";
					for ( int iVar = 0; iVar < sizeof(szVar)/sizeof(szVar[0]); iVar++ ) {
						TCHAR szSubBlock[ 256 ];
						_tcscpy( szSubBlock, szLang );
						_tcscat( szSubBlock, szVar[ iVar ] );
						if ( ::VerQueryValue( pvi, szSubBlock, &psfi, &nBytes ) )
							_tprintf( _T("\t%-20.20s: %s\n"), szVar[ iVar ], (LPTSTR)psfi );
					}
				}

				if ( bVerboseVersionInfo )
				{
					VS_FIXEDFILEINFO* pffi;
					if ( 0 == ::VerQueryValue( pvi, _T("\\"), reinterpret_cast<void**>(&pffi), &nBytes ) )
						cerr << "Unable to get fixed version info!" << endl;
					else {
						cout << "\nFixed info:" << hex << left <<
							"\n\t" << setw(20) << "Signature" << ": " << pffi->dwSignature <<
							"\n\t" << setw(20) << "StrucVersion" << ": " << pffi->dwStrucVersion <<
							"\n\t" << setw(20) << "FileVersionMS" << ": " << pffi->dwFileVersionMS <<
							"\n\t" << setw(20) << "FileVersionLS" << ": " << pffi->dwFileVersionLS <<
							"\n\t" << setw(20) << "ProductVersionMS" << ": " << pffi->dwProductVersionMS <<
							"\n\t" << setw(20) << "ProductVersionLS" << ": " << pffi->dwProductVersionLS <<
							"\n\t" << setw(20) << "FileFlagsMask" << ": " << pffi->dwFileFlagsMask <<
							"\n\t" << setw(20) << "FileFlags" << ": " << pffi->dwFileFlags <<
							"\n\t" << setw(20) << "FileOS" << ": " << pffi->dwFileOS <<
							"\n\t" << setw(20) << "FileType" << ": " << pffi->dwFileType <<
							"\n\t" << setw(20) << "FileSubtype" << ": " << pffi->dwFileSubtype <<
							"\n\t" << setw(20) << "FileDateMS" << ": " << pffi->dwFileDateMS <<
							"\n\t" << setw(20) << "FileDateLS" << ": " << pffi->dwFileDateLS <<
							endl;
					}
				}
			}
			delete [] pvi;
		}

	}
	return 0;
}
